@@ -1,5 +1,5 @@
 from textual.events import Event
 
 
-class Upd(Event, bubble=False):
+class DbUpdate(Event, bubble=True):
     pass
